"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const state = require("@codemirror/state");
const view = require("@codemirror/view");
const utils = require("@codemirror-toolkit/utils");
const ExtensionsCompartment = /* @__PURE__ */ new state.Compartment();
function updateExtensions(view2, extensions) {
  view2.dispatch({
    effects: ExtensionsCompartment.reconfigure(extensions)
  });
}
const ExtensionSetFacet = /* @__PURE__ */ state.Facet.define({
  combine(values) {
    return values.length ? values[0] : null;
  },
  static: true
});
function extensionManager(...initialExtensions) {
  return [
    ExtensionsCompartment.of(initialExtensions),
    ExtensionSetFacet.of(new Set(initialExtensions))
  ];
}
function wrapExtension(extension) {
  return { extension };
}
function getExtensionSet(state2) {
  const extensionSet = state2.facet(ExtensionSetFacet);
  if (!extensionSet) {
    throw new Error("extensionManager extension is not enabled");
  }
  return extensionSet;
}
function isExtensionArray(extension) {
  return Array.isArray(extension);
}
function flattenExtensions(extensions, depth) {
  if (depth <= 0) {
    return extensions;
  }
  return extensions.flatMap((extension) => {
    if (!isExtensionArray(extension)) {
      return extension;
    }
    return flattenExtensions(extension, depth - 1);
  });
}
function addExtension(view2, extension, { flattenDepth = 0 } = {}) {
  const extensionSet = getExtensionSet(view2.state);
  const flattenedExtensions = flattenExtensions([extension], flattenDepth);
  let hasAdded = false;
  for (const ext of flattenedExtensions) {
    if (!extensionSet.has(ext)) {
      extensionSet.add(ext);
      hasAdded = true;
    }
  }
  if (hasAdded) {
    updateExtensions(view2, [...extensionSet]);
  }
  return () => {
    removeExtension(view2, flattenedExtensions, { flattenDepth: 1 });
  };
}
function removeExtension(view2, extension, { flattenDepth = 0 } = {}) {
  const extensionSet = getExtensionSet(view2.state);
  const flattenedExtensions = flattenExtensions([extension], flattenDepth);
  let hasRemoved = false;
  for (const ext of flattenedExtensions) {
    if (extensionSet.delete(ext)) {
      hasRemoved = true;
    }
  }
  if (hasRemoved) {
    updateExtensions(view2, [...extensionSet]);
  }
}
function clearExtensions(view2) {
  const extensionSet = getExtensionSet(view2.state);
  if (extensionSet.size) {
    extensionSet.clear();
    updateExtensions(view2, []);
  }
}
function hasExtension(view2, extension) {
  const extensionSet = getExtensionSet(view2.state);
  return extensionSet.has(extension);
}
const ViewUpdateListenerEffect = /* @__PURE__ */ state.StateEffect.define();
const viewUpdateListenersField = /* @__PURE__ */ state.StateField.define({
  create() {
    return utils.createSetContainer();
  },
  update(listeners, transaction) {
    return transaction.effects.reduce(
      (resultListeners, effect) => effect.is(ViewUpdateListenerEffect) ? utils.mapStateEffectValue(
        effect,
        ({
          add: listenersToAdd = [],
          remove: listenersToRemove = [],
          clear: shouldClearListeners = false
        }) => {
          const clearedListeners = shouldClearListeners ? resultListeners.clear() : resultListeners;
          return clearedListeners.deleteMany(listenersToRemove).addMany(listenersToAdd);
        }
      ) : resultListeners,
      listeners
    );
  },
  provide(thisField) {
    return view.EditorView.updateListener.computeN([thisField], (state2) => {
      const listenerSetContainer = state2.field(thisField);
      return [...listenerSetContainer.extract()];
    });
  }
});
function viewUpdateListeners(...initialListeners) {
  return viewUpdateListenersField.init(
    () => utils.createSetContainer().addMany(initialListeners)
  );
}
function assertViewUpdateListenersField(view2) {
  if (!view2.state.field(viewUpdateListenersField, false)) {
    throw new Error("viewUpdateListeners extension is not enabled");
  }
}
function addViewUpdateListener(view2, listener) {
  assertViewUpdateListenersField(view2);
  view2.dispatch({
    effects: ViewUpdateListenerEffect.of({ add: [listener] })
  });
  return () => {
    removeViewUpdateListener(view2, listener);
  };
}
function removeViewUpdateListener(view2, listener) {
  assertViewUpdateListenersField(view2);
  view2.dispatch({
    effects: ViewUpdateListenerEffect.of({ remove: [listener] })
  });
}
function clearViewUpdateListeners(view2) {
  assertViewUpdateListenersField(view2);
  view2.dispatch({
    effects: ViewUpdateListenerEffect.of({ clear: true })
  });
}
exports.ViewUpdateListenerEffect = ViewUpdateListenerEffect;
exports.addExtension = addExtension;
exports.addViewUpdateListener = addViewUpdateListener;
exports.clearExtensions = clearExtensions;
exports.clearViewUpdateListeners = clearViewUpdateListeners;
exports.extensionManager = extensionManager;
exports.hasExtension = hasExtension;
exports.removeExtension = removeExtension;
exports.removeViewUpdateListener = removeViewUpdateListener;
exports.viewUpdateListeners = viewUpdateListeners;
exports.wrapExtension = wrapExtension;
//# sourceMappingURL=index.cjs.map
