import { Compartment, Facet, StateEffect, StateField } from "@codemirror/state";
import { EditorView } from "@codemirror/view";
import { createSetContainer, mapStateEffectValue } from "@codemirror-toolkit/utils";
const ExtensionsCompartment = /* @__PURE__ */ new Compartment();
function updateExtensions(view, extensions) {
  view.dispatch({
    effects: ExtensionsCompartment.reconfigure(extensions)
  });
}
const ExtensionSetFacet = /* @__PURE__ */ Facet.define({
  combine(values) {
    return values.length ? values[0] : null;
  },
  static: true
});
function extensionManager(...initialExtensions) {
  return [
    ExtensionsCompartment.of(initialExtensions),
    ExtensionSetFacet.of(new Set(initialExtensions))
  ];
}
function wrapExtension(extension) {
  return { extension };
}
function getExtensionSet(state) {
  const extensionSet = state.facet(ExtensionSetFacet);
  if (!extensionSet) {
    throw new Error("extensionManager extension is not enabled");
  }
  return extensionSet;
}
function isExtensionArray(extension) {
  return Array.isArray(extension);
}
function flattenExtensions(extensions, depth) {
  if (depth <= 0) {
    return extensions;
  }
  return extensions.flatMap((extension) => {
    if (!isExtensionArray(extension)) {
      return extension;
    }
    return flattenExtensions(extension, depth - 1);
  });
}
function addExtension(view, extension, { flattenDepth = 0 } = {}) {
  const extensionSet = getExtensionSet(view.state);
  const flattenedExtensions = flattenExtensions([extension], flattenDepth);
  let hasAdded = false;
  for (const ext of flattenedExtensions) {
    if (!extensionSet.has(ext)) {
      extensionSet.add(ext);
      hasAdded = true;
    }
  }
  if (hasAdded) {
    updateExtensions(view, [...extensionSet]);
  }
  return () => {
    removeExtension(view, flattenedExtensions, { flattenDepth: 1 });
  };
}
function removeExtension(view, extension, { flattenDepth = 0 } = {}) {
  const extensionSet = getExtensionSet(view.state);
  const flattenedExtensions = flattenExtensions([extension], flattenDepth);
  let hasRemoved = false;
  for (const ext of flattenedExtensions) {
    if (extensionSet.delete(ext)) {
      hasRemoved = true;
    }
  }
  if (hasRemoved) {
    updateExtensions(view, [...extensionSet]);
  }
}
function clearExtensions(view) {
  const extensionSet = getExtensionSet(view.state);
  if (extensionSet.size) {
    extensionSet.clear();
    updateExtensions(view, []);
  }
}
function hasExtension(view, extension) {
  const extensionSet = getExtensionSet(view.state);
  return extensionSet.has(extension);
}
const ViewUpdateListenerEffect = /* @__PURE__ */ StateEffect.define();
const viewUpdateListenersField = /* @__PURE__ */ StateField.define({
  create() {
    return createSetContainer();
  },
  update(listeners, transaction) {
    return transaction.effects.reduce(
      (resultListeners, effect) => effect.is(ViewUpdateListenerEffect) ? mapStateEffectValue(
        effect,
        ({
          add: listenersToAdd = [],
          remove: listenersToRemove = [],
          clear: shouldClearListeners = false
        }) => {
          const clearedListeners = shouldClearListeners ? resultListeners.clear() : resultListeners;
          return clearedListeners.deleteMany(listenersToRemove).addMany(listenersToAdd);
        }
      ) : resultListeners,
      listeners
    );
  },
  provide(thisField) {
    return EditorView.updateListener.computeN([thisField], (state) => {
      const listenerSetContainer = state.field(thisField);
      return [...listenerSetContainer.extract()];
    });
  }
});
function viewUpdateListeners(...initialListeners) {
  return viewUpdateListenersField.init(
    () => createSetContainer().addMany(initialListeners)
  );
}
function assertViewUpdateListenersField(view) {
  if (!view.state.field(viewUpdateListenersField, false)) {
    throw new Error("viewUpdateListeners extension is not enabled");
  }
}
function addViewUpdateListener(view, listener) {
  assertViewUpdateListenersField(view);
  view.dispatch({
    effects: ViewUpdateListenerEffect.of({ add: [listener] })
  });
  return () => {
    removeViewUpdateListener(view, listener);
  };
}
function removeViewUpdateListener(view, listener) {
  assertViewUpdateListenersField(view);
  view.dispatch({
    effects: ViewUpdateListenerEffect.of({ remove: [listener] })
  });
}
function clearViewUpdateListeners(view) {
  assertViewUpdateListenersField(view);
  view.dispatch({
    effects: ViewUpdateListenerEffect.of({ clear: true })
  });
}
export {
  ViewUpdateListenerEffect,
  addExtension,
  addViewUpdateListener,
  clearExtensions,
  clearViewUpdateListeners,
  extensionManager,
  hasExtension,
  removeExtension,
  removeViewUpdateListener,
  viewUpdateListeners,
  wrapExtension
};
//# sourceMappingURL=index.js.map
